<div class='faq_sharing'>
<div class='question opened collapsible'>
  <a class='toggle' href='#'>
    <h4>{{ add_to_aspect_q }}</h4>
  </a>
  <div class='answer hideable'>
    {{ add_to_aspect_a1 }}
    <ul>
      <li>{{ add_to_aspect_li1 }}</li>
      <li>{{ add_to_aspect_li2 }}</li>
      <li>{{ add_to_aspect_li3 }}</li>
      <li>{{ add_to_aspect_li4 }}</li>
      <li>{{ add_to_aspect_li5 }}</li>
      <li>{{ add_to_aspect_li6 }}</li>
      <li>{{ add_to_aspect_li7 }}</li>
      <li>{{ add_to_aspect_li8 }}</li>
    </ul>
    {{ add_to_aspect_a2 }}
  </div>
</div>
<div class='question opened collapsible'>
  <a class='toggle' href='#'>
    <h4>{{ sharing_notification_q }}</h4>
  </a>
  <div class='answer hideable'>{{ sharing_notification_a }}</div>
</div>
<div class='question opened collapsible'>
  <a class='toggle' href='#'>
    <h4>{{ only_sharing_q }}</h4>
  </a>
  <div class='answer hideable'>{{ only_sharing_a }}</div>
</div>
<div class='question opened collapsible'>
  <a class='toggle' href='#'>
    <h4>{{ list_not_sharing_q }}</h4>
  </a>
  <div class='answer hideable'>{{ list_not_sharing_a }}</div>
</div>
<div class='question opened collapsible'>
  <a class='toggle' href='#'>
    <h4>{{ see_old_posts_q }}</h4>
  </a>
  <div class='answer hideable'>{{ see_old_posts_a }}</div>
</div>
</div>