<div class="conversation-recipient-tag clearfix" data-diaspora-handle="{{ handle }}">
  <div href="{{ url }}">
    <img src="{{ avatar }}" class="avatar img-responsive center-block">
  </div>
  <div class="pull-left clearfix name-and-handle" href="{{ url }}">
    <div class="name">{{ name }}</div>
    <div class="diaspora-id">{{ handle }}</div>
  </div>
  <div class="remove pull-right clearfix">
    <i class="entypo-circled-cross"></i>
  </div>
</div>
