// @license magnet:?xt=urn:btih:0b31508aeb0634b347b8270c7bee4d411b5d4109&dn=agpl-3.0.txt AGPL-v3-or-Later

app.models.PollParticipation = Backbone.Model.extend({
  urlRoot: function(){
    return '/posts/' + this.get('post_id') + "/poll_participations";
  }
});
// @license-end

