.page-contacts {
  .sidebar { padding-bottom: 10px; }

  .stream.contacts .header {
    border-bottom: 1px solid $border-grey;
    min-height: 55px;
    form { margin: 0; }
    input,
    .btn {
      margin-bottom: 11px;
      margin-top: 11px;
    }
  }

  .stream.contacts .aspect-controls {
    margin-bottom: 7px;
    margin-left: 30px;
    margin-right: -10px;
    margin-top: 7px;
  }
}

#contacts_container {
  #people-stream.contacts {
    .header {
      #change_aspect_name { cursor: pointer; }
      #aspect_name_form {
        display: none;
      }
      #contact_list_search {
        margin: 11px 0 0;
        width: 150px;
        &:focus { width: 250px; }
      }
      #aspect_controls > .contacts_button {
        cursor: pointer;
        text-decoration: none;
        margin-right: 25px;
      }
      #chat_privilege_toggle > .enabled {
        color: $text-color-active;
      }
      .contacts-header-icon {
        font-size: 24.5px;
        line-height: 40px;
        color: $text-color-pale;
        &:hover { color: $text-color; }
      }
      #suggest_member.btn { margin-top: 8px; }
    }

    .stream-element {
      .contact_remove-from-aspect, .contact_add-to-aspect {
        text-decoration: none;
        cursor: pointer;
        font-size: 20px;
        line-height: 50px;
        margin: 0 10px;
        color: $text-color-pale;
        &:hover { color: $text-color; }
      }
      &.in_aspect {
        border-left: 3px solid $brand-success;
        background-color: lighten($brand-success,35%);
      }
      &:not(.in_aspect) { border-left: 3px solid $framed-background; }
    }

    .no_contacts {
      text-align: center;
      margin-top: 50px;
    }
    .well { margin: 10px; }
  }
}

#aspect_nav {
  li.aspect > a { padding-left: 30px; }
  li.new_aspect > a { padding-left: 30px; }
  li.aspect.ui-sortable-handle.ui-sortable-helper { background: #FFF; }
}

#no_contacts {
  margin-top: 20px;
  text-align: center;
}
