<div class="media {{#if showPost}} {{#if nsfw}} shield-off {{/if}} {{else}} shield-active {{/if}}">

  {{#unless preview}}
    {{#with author}}
      <a href="/people/{{guid}}" class="img {{{hovercardable this}}}">
        {{{personImage this}}}
      </a>
    {{/with}}
  {{/unless}}

  <div class="bd">
    {{#unless preview}}
      {{#if loggedIn}}
        <div class="post-controls"></div>
      {{/if}}

      <div>
        {{#linkToAuthor author}}
          {{~name~}}
        {{/linkToAuthor}}

        <span class="details gray post-timestamp">
          -
          <a href="/posts/{{id}}">
            <time class="timeago" data-original-title="{{{localTime created_at}}}" datetime="{{created_at}}" />
          </a>

          <a href="/posts/{{guid}}" class="permalink" title="{{t "stream.permalink"}}">
            <i class="entypo-link"></i>
          </a>
        </span>
      </div>
    {{/unless}}

    <div class="post-content"> </div>
    <div class="status-message-location nsfw-hidden"> </div>

    {{#unless preview}}
      <div class="feedback nsfw-hidden"> </div>
      <div class="likes nsfw-hidden"> </div>
      <div class="reshares nsfw-hidden"> </div>
      <div class="comments nsfw-hidden"> </div>
    {{/unless}}
  </div>
</div>
