<div class='faq_getting_help'>
<h1>diaspora* FAQ</h1>
<br>
<div class='question opened'>
  <h4>{{{ getting_started_q }}}</h4>
  <div class='answer'>
    {{{ getting_started_a }}}
    </div>
</div>
<div class='question opened'>
  <h4>{{{ get_support_q }}}</h4>
  <div class='answer'>
    <ul>
      <li>{{{ get_support_a_website }}}</li>
      <li>{{{ get_support_a_tutorials }}}</li>
      <li>{{{ get_support_a_wiki }}}</li>
      <li>{{{ get_support_a_irc }}}</li>
      <li>{{{ get_support_a_faq }}}</li>
      <li>{{{ get_support_a_hashtag }}}</li>
      <li>{{{ get_support_a_discourse }}}</li>
    </ul>
  </div>
</div>
</div>
