
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ keyboard_shortcuts_q }}</h4>
    </a>
    <div class='answer hideable'>
      {{ keyboard_shortcuts_a1 }}
      <ul>
        <li>{{ keyboard_shortcuts_li1 }}</li>
        <li>{{ keyboard_shortcuts_li2 }}</li>
        <li>{{ keyboard_shortcuts_li3 }}</li>
        <li>{{ keyboard_shortcuts_li4 }}</li>
        <li>{{ keyboard_shortcuts_li5 }}</li>
        <li>{{ keyboard_shortcuts_li6 }}</li>
        <li>{{ keyboard_shortcuts_li7 }}</li>
        <li>{{ keyboard_shortcuts_li8 }}</li>
      </ul>
    </div>
  </div>
