<span class="post_scope gray">
  {{#if public}}
    {{t "stream.public"}}
  {{else}}
    {{t "stream.limited"}}
  {{/if}}

  {{#if provider_display_name}}
    <strong>
      {{t "stream.via" provider=provider_display_name}}
    </strong>
  {{/if}}
  –
</span>

<a href="#" class="like" rel='nofollow'>
  {{~#if userLike~}}
    {{~t "stream.unlike"~}}
  {{~else~}}
    {{~t "stream.like"~}}
  {{~/if~}}
</a>
·
{{#if userCanReshare}}
  <a href="#" class="reshare" rel='nofollow'>{{t "stream.reshare"}}</a>
  ·
{{/if}}

<a href="#" class="focus_comment_textarea" rel="nofollow">{{t "stream.comment"}}</a>
