/*   Copyright (c) 2010-2011, Diaspora Inc.  This file is
 *   licensed under the Affero General Public License version 3 or later.  See
 *   the COPYRIGHT file.
 */
//= require js_image_paths
//= require js-routes
//= require underscore
//= require backbone
//= require autosize
//= require charcount
//= require jquery-placeholder
//= require jquery.timeago
//= require jquery.ui/ui/core
//= require jquery.ui/ui/widget
//= require jquery.ui/ui/mouse
//= require jquery.ui/ui/sortable
//= require keycodes
//= require jquery.autoSuggest.custom
//= require fine-uploader/fine-uploader.core
//= require handlebars.runtime
//= require_tree ../templates
//= require posix-bracket-expressions
//= require markdown-it
//= require markdown-it-diaspora-mention
//= require markdown-it-for-inline
//= require markdown-it-hashtag
//= require markdown-it-sanitizer
//= require markdown-it-sub
//= require markdown-it-sup
//= require markdown-it-html5-embed
//= require highlightjs
//= require clear-form
//= require corejs-typeahead
//= require app/app
//= require diaspora
//= require_tree ./helpers
//= require_tree ./pages
//= require_tree ./widgets
//= require bootstrap
//= require osmlocator
//= require bootstrap-switch
//= require blueimp-gallery
//= require blueimp-gallery/blueimp-gallery-indicator
//= require leaflet
//= require api/authorization_page
//= require bootstrap-markdown/bootstrap-markdown
//= require helpers/markdown_editor
//= require helpers/protocol_handler
//= require jquery.are-you-sure
