<div class='question opened collapsible'>
  <a class='toggle' href='#'>
    <h4>{{ what_are_tags_for_q }}</h4>
  </a>
  <div class='answer hideable'>{{{ what_are_tags_for_a }}}</div>
</div>
<div class='question opened collapsible'>
  <a class='toggle' href='#'>
    <h4>{{ tags_in_comments_q }}</h4>
  </a>
  <div class='answer hideable'>{{{ tags_in_comments_a }}}</div>
</div>
<div class='question opened collapsible'>
  <a class='toggle' href='#'>
    <h4>{{ followed_tags_q }}</h4>
  </a>
  <div class='answer hideable'>{{{ followed_tags_a }}}</div>
</div>
<div class='question opened collapsible'>
  <a class='toggle' href='#'>
    <h4>{{ people_tag_page_q }}</h4>
  </a>
  <div class='answer hideable'>{{{ people_tag_page_a }}}</div>
</div>
<div class='question opened collapsible'>
  <a class='toggle' href='#'>
    <h4>{{ filter_tags_q }}</h4>
  </a>
  <div class='answer hideable'>{{{ filter_tags_a }}}</div>
</div>