body > .container-fluid.chat-roster-shown {
  padding-right: 224px;
  #back-to-top { right: 244px; }
}
body > .container-fluid.chat-roster-hidden {
  #back-to-top { right: 54px; }
}

// This element is instanciated by JSXC. Does not have to follow naming conventions
// scss-lint:disable IdSelector, SelectorFormat
#jsxc_roster {
  top: $navbar-height;
}
// scss-lint:enable IdSelector, SelectorFormat
