<div class="show_comments comment {{#unless showExpandCommentsLink}} hidden {{/unless}}">
  <div class="media">
    <a href="/posts/{{id}}#comments" class="toggle_post_comments">
      {{t "stream.more_comments" count=moreCommentsCount}}
    </a>
  </div>
</div>

<div class="loading-comments comment text-center hidden">
  <div class="media">
    <div class="loader">
      <div class="spinner"></div>
    </div>
  </div>
</div>

<div class="comments"> </div>

{{#if loggedIn}}
  <div class="comment media new-comment-form-wrapper {{#unless commentsCount}} hidden {{/unless}}">
    {{#with current_user}}
      <a href="/people/{{guid}}" class="img">
        {{{personImage this}}}
      </a>
    {{/with}}

    <div class="bd">
      <form accept-charset="UTF-8" action="/posts/{{id}}/comments"
            class="new-comment" id="new-comment-on-{{id}}" method="post">

        <textarea class="comment-box form-control mention-textarea"
                  id="comment_text_on_{{id}}" name="text" rows="1" required placeholder="{{t "stream.comment"}}" />
        <div class="typeahead-mention-box-wrap">
          <input class="typeahead-mention-box hidden" type="text">
        </div>

        <div class="submit-button">
          <input class="btn btn-primary" id="comment_submit_{{id}}" name="commit" type="submit" value="{{t "stream.comment"}}" />
        </div>
      </form>
    </div>
  </div>
{{/if}}
