<div class="container">

  <div id="section_header">
    <h2 class="help_header">{{ title_header }}</h2>
  </div>

  <div class="row">
    <div class="col-md-3">
      <div id='faq_nav'>
        <ul>
          <li>
            <a href="#" class="section-unselected faq-link-getting-help" data-section="getting_help" data-items="">{{ title_getting_help }}</a>
            <span class="section-selected">{{ title_getting_help }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link" data-section="account_and_data_management" data-items="move_pods download_data close_account data_visible_to_podmin data_other_podmins">{{ title_account_and_data_management }}</a>
            <span class="section-selected">{{ title_account_and_data_management }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link" data-section="aspects" data-items="what_is_an_aspect who_sees_post contacts_know_aspect person_multiple_aspects remove_notification change_aspect_of_post post_multiple_aspects restrict_posts_i_see rename_aspect delete_aspect">{{ title_aspects }}</a>
            <span class="section-selected">{{ title_aspects }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link" data-section="mentions" data-items="what_is_a_mention how_to_mention mention_in_comment see_mentions">{{ title_mentions }}</a>
            <span class="section-selected">{{ title_mentions }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link" data-section="pods" data-items="what_is_a_pod find_people use_search_box">{{ title_pods }}</a>
            <span class="section-selected">{{ title_pods }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link-posts-and-posting" data-section="posts_and_posting" data-items="hide_posts ignore_user format_text insert_images">{{ title_posts_and_posting }}</a>
            <span class="section-selected">{{ title_posts_and_posting }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link" data-section="private_posts" data-items="who_sees_post can_comment can_reshare see_comment">{{ title_private_posts }}</a>
            <span class="section-selected">{{ title_private_posts }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link" data-section="public_posts" data-items="who_sees_post find_public_post can_comment_reshare_like see_comment_reshare_like deselect_aspect_posting">{{ title_public_posts }}</a>
            <span class="section-selected">{{ title_public_posts }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link" data-section="resharing_posts" data-items="reshare_public_post_aspects reshare_private_post_aspects">{{ title_resharing_posts }}</a>
            <span class="section-selected">{{ title_resharing_posts }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link" data-section="profile" data-items="whats_in_profile who_sees_profile what_do_tags_do">{{ title_profile }}</a>
            <span class="section-selected">{{ title_profile }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link-sharing" data-section="sharing" data-items="">{{ title_sharing }}</a>
            <span class="section-selected">{{ title_sharing }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link-tags" data-section="tags" data-items="what_are_tags_for tags_in_comments followed_tags people_tag_page filter_tags">{{ title_tags }}</a>
            <span class="section-selected">{{ title_tags }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link-keyboard-shortcuts" data-section="keyboard_shortcuts" data-items="">{{ title_keyboard_shortcuts }}</a>
            <span class="section-selected">{{ title_keyboard_shortcuts }}</span>
          </li>
          <li>
            <a href="#" class="section-unselected faq-link" data-section="miscellaneous" data-items="back_to_top photo_albums subscribe_feed diaspora_app">{{ title_miscellaneous }}</a>
            <span class="section-selected">{{ title_miscellaneous }}</span>
          </li>
          {{#if chat_enabled }}
          <li>
            <a href="#" class="section-unselected faq-link-chat" data-section="chat" data-items="add_contact_roster i_m_a_podmin">{{ title_chat }}</a>
            <span class="section-selected">{{ title_chat }}</span>
          </li>
          {{/if}}
        </ul>
      </div>
    </div>

    <div class="col-md-9 last" id=faq>
    </div>
  </div>
</div>
