# frozen_string_literal: true

module Stream
  require 'stream/activity'
  require 'stream/aspect'
  require 'stream/comments'
  require 'stream/followed_tag'
  require 'stream/likes'
  require 'stream/mention'
  require 'stream/multi'
  require 'stream/person'
  require 'stream/public'
  require 'stream/tag'
end
