//= require jquery3
//= require handlebars.runtime
//= require main
//= require fine-uploader/fine-uploader.core
//= require mobile/mobile
//= require jquery.autoSuggest.custom
//= require contact-list
//= require sinon
//= require jasmine-ajax
