#hovercard {
  border-radius: 2px;
  box-shadow: 0 0 5px #666666;

  position: relative;
  display: inline-block;
  min-width: 250px;
  max-width: 400px;

  background-color: $hovercard-background;
  border: 1px solid $border-dark-grey;
  font-size: small;

  .avatar {
    float: left;
    height: 70px;
    object-fit: cover;
    width: 70px;
  }

  $image_width: 80px; /* including margin */

  > h4,
  > div:not(.card-footer) {
    margin-left: $image_width;
  }

  > h4,
  > div,
  .hashtags {
    overflow: hidden;
    white-space: nowrap;
    text-overflow: ellipsis;
  }

  #hovercard_dropdown_container {
    overflow: visible !important; /* otherwise the aspect dropdown is cropped */
  }

  padding: 5px {
    bottom: 25px;
  };

  h4 {
    margin-top: 0px;
    margin-bottom: 0px;
    padding-bottom: 0px;
    font-size: 16px;
    a {
      font-weight: bold !important;
    }
  }

  .handle {
    color: $text-grey;
    line-height: 18px;
    padding-top: 0px;
    margin-top: 0px;
    margin-bottom: 5px;
  }

  .btn-group.aspect-membership-dropdown { margin: 0 !important; }
}

#hovercard_container {
  position: absolute;
  display: none;
  z-index: 2000;

  min-width: 250px;
  max-width: 400px;

  padding: 10px {
    top: 5px;
  }
}
