.media {
  margin: 10px;
}

.media,
.bd {
  overflow: hidden;
  _overflow: visible;
  zoom: 1;
}

.media .img {
  float: left;
  margin-right: 10px;

  img { display: block; }
}

.media .imgEt { float: right; margin-left: 10px; }
