# frozen_string_literal: true

#   Copyright (c) 2010-2011, Diaspora Inc.  This file is
#   licensed under the Affero General Public License version 3 or later.  See
#   the COPYRIGHT file.

module Diaspora
  module Federated
    require "diaspora/federated/base"
    require "diaspora/federated/retraction"
    require "diaspora/federated/contact_retraction"
  end
end
