.page-sessions.action-new,
.page-passwords.action-new,
.page-passwords.action-edit {
  padding-top: 25px;

  .logos-asterisk {
    height: 154px;
    margin: auto;
    margin-bottom: 12px;
    width: 154px;
  }

  .block-form {
    max-width: 400px;
  }
}
