{{#if authorIsCurrentUser}}
  <a href="#" rel="nofollow" class="delete remove_post" title="{{t "delete"}}">
    <i class="entypo-trash"></i>
  </a>
{{else}}
  {{#if participation}}
    <a href="#" rel="nofollow" class="destroy_participation" title="{{t "stream.disable_post_notifications"}}">
      <i class="entypo-bell"></i>
    </a>
  {{else}}
    <a href="#" rel="nofollow" class="create_participation" title="{{t "stream.enable_post_notifications"}}">
      <i class="entypo-bell"></i>
    </a>
  {{/if}}
  <a href="#" rel="nofollow" data-type="Post" class="post_report" title="{{t "report.name"}}">
    <i class="entypo-warning"></i>
  </a>
  <a href="#" rel="nofollow" class="block_user" title="{{t "ignore"}}">
    <i class="entypo-block"></i>
  </a>
  <a href="#" rel="nofollow" class="delete hide_post" title="{{t "stream.hide"}}">
    <i class="entypo-cross"></i>
  </a>
{{/if}}
