<div id="single-post-container">
  <div class='row'>
    <div id='single-post-content' class='col-md-6 single-post-content'>
    </div>
    <div id='single-post-interactions' class='col-md-6 single-post-interactions'>
    </div>
  </div>
</div>
