<div id="author_info" class="row">
  <div class="col-sm-10">
    <h2>
      <span id="name">{{name}}</span>
      <span id="diaspora_handle">{{diaspora_id}}</span>
      {{#if show_profile_btns}}
        {{{sharingMessage this}}}
      {{/if}}
    </h2>

    {{#if has_tags}}
      <div class="description">
        <i class="entypo-tag"></i>
        {{fmtTags profile.tags}}
      </div>
    {{else}}
      {{#if is_own_profile}}
        <div class="description">
          <i>{{t 'profile.you_have_no_tags'}}</i>
          <span class="add_tags">
            <a href="{{urlTo 'editProfile'}}">{{t 'profile.add_some'}}</a>
          </span>
        </div>
      {{/if}}
    {{/if}}
  </div>
  <div class="clearfix col-sm-2">
    {{#if loggedIn}}
      <div class="pull-right">
        {{#if is_own_profile}}
        {{!-- can't block myself, so don't check it here --}}
          <a href="{{urlTo 'editProfile'}}" id="edit_profile" class="btn btn-primary">{{t 'people.edit_my_profile'}}</a>
        {{else}} {{#if is_blocked}}
          <a href="#" id="unblock_user_button" class="btn btn-danger">{{t 'people.stop_ignoring'}}</a>
        {{else}}
          <div class="placeholder aspect-membership-dropdown"></div>
        {{/if}}{{/if}}
      </div>
    {{/if}}
  </div>
</div>

<div id="profile-horizontal-bar">
  {{#if show_profile_btns}}
    <div id="profile_buttons" class="pull-right">
      {{#if is_receiving}}
        {{!-- create status message with mention --}}
        <span class="profile_button">
          <span id="mention_button" class="profile-header-icon" title="{{t 'people.mention'}}" data-placement="bottom" data-toggle="modal">@</span>
        </span>
      {{/if}}

      {{#if is_mutual}}
        {{!-- create private conversation with person --}}
        <span class="profile_button">
          <i id="message_button" class="entypo-mail profile-header-icon" title="{{t 'people.message'}}" data-placement="bottom" data-toggle="modal"></i>
        </span>
      {{/if}}

      {{#unless is_blocked}}
        {{!-- ignore the person --}}
        <a href="#" class="profile_button" rel="nofollow">
          <i id="block_user_button" class="entypo-block profile-header-icon block_user" title="{{t 'ignore'}}" data-placement="bottom"></i>
        </a>
      {{/unless}}
    </div>
  {{/if}}

  <ul id="profile_nav">
    <li {{#isCurrentProfilePage guid diaspora_id}} class="active" {{/isCurrentProfilePage}}>
      <a href="{{urlTo 'person' guid}}" id="posts_link">
        <i class="entypo-docs"></i>
        {{t 'profile.posts'}}
      </a>
    </li>
    {{#if show_photos}}
      <li {{#isCurrentPage 'personPhotos' guid}} class="active" {{/isCurrentPage}}>
        <a href="{{urlTo 'personPhotos' guid}}" id="photos_link">
          <i class="entypo-picture"></i>
          {{t 'profile.photos'}}
          <div class="badge badge-default">{{photos}}</div>
        </a>
      </li>
    {{/if}}
  </div>
</div>
