<div class="nsfw-shield">
  <strong>
    #NSFW
  </strong>
  |
  <a href="#" class="show_nsfw_post">
    {{t "stream.show_nsfw_post"}}
  </a>
  |
  <a href="#" class="toggle_nsfw_state">
    {{t "stream.show_nsfw_posts"}}
  </a>
</div>

<div class="nsfw-off">
  <strong>
    #NSFW
  </strong>
  |
  <a href="#" class="toggle_nsfw_state">
      {{t "stream.hide_nsfw_posts"}}
  </a>
</div>

{{#if largePhoto}}
  <div class="photo-attachments nsfw-hidden">
    {{#with largePhoto}}
      <a href="{{sizes.raw}}" class="stream-photo-link gallery-picture">
        <img src="{{sizes.large}}" class="stream-photo big_stream_photo">
      </a>
    {{/with}}

    {{#each smallPhotos}}
      <a href="{{sizes.raw}}" class="stream-photo-link gallery-picture">
        <img src="{{sizes.small}}" class="stream-photo thumb_small">
      </a>
    {{/each}}
  </div>
{{/if}}

<div class="collapsible nsfw-hidden">
  <div class="markdown-content">
    {{#if text}}
      {{{text}}}
    {{else}}
      {{#unless largePhoto}}
        {{#if isReshare}}
          <div class="reshare">
            <p>
              {{t "stream.original_post_deleted"}}
            </p>
          </div>
        {{/if}}
      {{/unless}}
    {{/if}}
  </div>
  <div class="oembed"></div>
  <div class="opengraph"></div>
  <div class="poll"></div>
</div>
