#single-post-container {
  padding-top: 20px;
}

#single-post-content {
  .head {
    border-bottom: 1px solid $border-grey;
    padding: 10px 0;
    #post-info {
      .info {
        color: lighten($text-grey,10%);
        font-size: 12px;
        .post-time a { color: $text-grey; }
        .post_scope { margin-right: 5px; }
        .status-message-location {
          padding-top: 2px;
          line-height: $font-size-base;
        }
      }
      .bd {
        padding-left: 10px;
      }
    }

    .author-name { color: $grey; }

    .near-from {
      color: $text-grey;
      font-size: 12px;
      margin: 10px 15px 0;
    }
    .mapContainer {
      margin: 10px 15px 0;
    }
    .reshare.row {
      border-top: 1px solid lighten($border-grey,5%);
      padding-top: 10px;
      margin-top: 10px;
    }
    #reshare-info {
      line-height: 15px;
      i.entypo-reshare {
        color: $text-dark-grey;
        font-size: 28px;
        line-height: 30px;
        margin-left: 7px;
        margin-right: 7px;
      }
      .post-context {
        font-size: 12px;

        #single-post-moderation {
          margin-left: 5px;

          > div.control-icons {
            display: inline-block;
          }
        }
      }
      .post-time a {
        color: $text-grey;
        font-size: 12px;
      }
      .avatar {
        height: 30px;
        width: 30px;
      }
      .img { margin-right: 10px; }
    }
    #single-post-actions {
      i {
        font-size: 28px;
        line-height: 30px;
      }
      i.entypo-comment:hover {
        color: #424242;
      }
      .post_report i.gray:hover {
        color: $red;
      }
      i.entypo-heart.gray:hover {
        color: $red;
      }
      i.entypo-heart.red:hover {
        color: #f55f5a;
      }
      i.entypo-reshare:hover {
        color: $blue;
      }
      time {
        float: right;
        margin-left: 3px;
      }
      a {
        margin: 0 0 0 6px;
        &:hover {
          text-decoration: none;
        }
      }
    }
  }

  .body {
    padding: 20px 15px;

    .nsfw-off { display: none; }
    .nsfw-shield { display: none; }
  }
}

#single-post-interactions {

  > .framed-content {
    padding-top: 10px;
  }

  .no-comments { text-align: center; }

  a {
    color: $link-color;
  }
  .count {
    float: left;
    i {
      display: inline-block;
      text-align: center;
      width: 25px;
      vertical-align: middle;
    }
    span {
      margin-left: -3px;
      width: 30px;
      display: inline-block;
      text-align: center;
    }
  }
  #reshares, #likes, #comments-meta {
    margin-left: 7px;
    margin-bottom: 8px;
    img{ display: inline; }
  }

  .comments > .comment,
  .comment.new-comment-form-wrapper {
    padding: 10px;
  }

  .count,
  .interaction-avatars {
    line-height: 25px;
  }

  .interaction-avatars {
    overflow: hidden;

    .author-name:focus,
    .author-name:hover {
      text-decoration: none;
    }

    .loader {
      height: $line-height-computed;
      vertical-align: text-bottom;
      width: $line-height-computed;
    }
  }
}
