<div class='question opened collapsible'>
    <a class='toggle' href='#'>
        <h4>{{ add_contact_roster_q }}</h4>
    </a>
    <div class='answer hideable'>{{{ add_contact_roster_a }}}</div>
</div>