/* AutoSuggest CSS - Version 1.2 */

ul.as-selections {
  list-style-type: none;
  margin: 0;
  overflow: auto;
  background-color: #fff;
  border-radius: 3px;
  width: 100%;
  padding: 0;
}

ul.as-selections li {
  float: left;
  margin: 1px 4px 1px 0;
  cursor: pointer;
}

ul.as-selections li.as-selection-item {
  color: #2b3840;
  font-size: 13px;
  text-shadow: 0 1px 1px #fff;
  background-color: #ddeefe;
  background-image: gradient(linear, 0% 0%, 0% 100%, from(#ddeefe), to(#bfe0f1));
  border: 1px solid #acc3ec;
  padding: 0;
  padding-top: 6px;
  padding-bottom: 6px;
  padding-left: 6px;
  border-radius: 5px;
  box-shadow: 0 1px 1px #e4edf2;
  line-height: 10px;
  margin-bottom: 10px;
}

ul.as-selections li.as-selection-item:last-child {
  margin-left: 30px;
}

ul.as-selections li.as-selection-item a.as-close {
  float: right;
  margin: 0px 3px 0 0px;
  padding: 0 3px;
  cursor: pointer;
  color: #5491be;
  font-family: "Helvetica", helvetica, arial, sans-serif;
  font-size: 14px;
  font-weight: bold;
  text-shadow: 0 1px 1px #fff;
  transition: color .1s ease-in;
}

ul.as-selections li.as-selection-item.blur {
  color: #666666;
  background-color: #f4f4f4;
  background-image: gradient(linear, 0% 0%, 0% 100%, from(#f4f4f4), to(#d5d5d5));
  border-color: #bbb;
  border-top-color: #ccc;
  box-shadow: 0 1px 1px #e9e9e9;
}

ul.as-selections li.as-selection-item.blur a.as-close {
  color: #999;
}

ul.as-selections li:hover.as-selection-item {
  color: #2b3840;
  background-color: #bbd4f1;
  background-image: gradient(linear, 0% 0%, 0% 100%, from(#bbd4f1), to(#a3c2e5));
  border-color: #6da0e0;
  border-top-color: #8bb7ed;
}

ul.as-selections li:hover.as-selection-item a.as-close {
  color: #4d70b0;
}

ul.as-selections li.as-selection-item.selected {
  border-color: #1f30e4;
}

ul.as-selections li.as-selection-item a:hover.as-close {
  color: #1b3c65;
}

ul.as-selections li.as-selection-item a:active.as-close {
  color: #4d70b0;
}

ul.as-selections li.as-original {
  width: 100%;
  margin: 0;
  list-style: none;
  padding: 0;
}

ul.as-selections li.as-original input {
  outline: none;
  font-size: 13px;
  margin: 0;
  line-height: 20px;
  width: 100%;
}

ul.as-list {
  position: absolute;
  list-style-type: none;
  list-style: none;
  margin: 2px 0 0 0;
  padding: 0;
  font-size: 13px;
  color: #000;
  background-color: #fff;
  background-color: rgba(255,255,255,0.95);
  z-index: 2;
  box-shadow: 0 2px 12px #222;
  border-radius: 5px;
}

li.as-result-item, li.as-message {
  margin: 0 0 0 0;
  padding: 5px;
  background-color: transparent;
  border: 1px solid #fff;
  border-bottom: 1px solid #ddd;
  cursor: pointer;
  border-radius: 3px;
}

li:first-child.as-result-item {
  margin: 0;
}

li.as-message {
  margin: 0;
  cursor: default;
}

li.as-result-item.active {
  background-color: #3668d9;
  background-image: gradient(linear, 0% 0%, 0% 64%, from(rgb(110, 129, 245)), to(rgb(62, 82, 242)));
  border-color: #3342e8;
  color: #fff;
  text-shadow: 0 1px 2px #122042;
}

li.as-result-item em {
  font-style: normal;
  background: #444;
  padding: 0 2px;
  color: #fff;
}

li.as-result-item.active em {
  background: #253f7a;
  color: #fff;
}

/* Opera Hacks  */
@media all and (-webkit-min-device-pixel-ratio:10000), not all and (-webkit-min-device-pixel-ratio:0) {
  ul.as-list {
    border: 1px solid #ccc;
  }
  ul.as-selections li.as-selection-item a.as-close {
    margin-left: 4px;
    margin-top: 0;
  }
}

/* IE Hacks  */
ul.as-list {
  border: 1px solid #ccc\9;
}
ul.as-selections li.as-selection-item a.as-close {
  margin-left: 4px\9;
  margin-top: 0\9;
}

/* Firefox 3.0 Hacks */
ul.as-list,  x:-moz-any-link, x:default {
  border: 1px solid #ccc;
}
BODY:first-of-type ul.as-list, x:-moz-any-link, x:default { /* Target FF 3.5+ */
  border: none;
}
