<div id="{{guid}}">
  <div class="img">
    {{#linkToAuthor author}}
      {{{personImage this "small" "small"}}}
    {{/linkToAuthor}}
  </div>

  <div class="bd">
    <div class="control-icons">
    {{#if loggedIn}}
      {{#if canRemove}}
        <a href="#" class="delete comment_delete" title="{{t "delete"}}">
          <i class="entypo-trash"></i>
        <a/>
      {{else}}
        <a href="#" data-type="Comment" class="comment_report" title="{{t "report.name"}}">
          <i class="entypo-warning"></i>
        </a>
      {{/if}}
    {{/if}}
    </div>

    <div>
      {{#linkToAuthor author}}
        {{name}}
      {{/linkToAuthor}}
      -
      <a href="/posts/{{parent.id}}#{{guid}}" class="permalink_comment">
        <time class="timeago" data-original-title="{{{localTime created_at}}}" datetime="{{created_at}}"/>
      </a>
      <a href="/posts/{{parent.guid}}#{{guid}}" class="permalink gray" title="{{t "stream.permalink"}}">
        <i class="entypo-link"></i>
      </a>
    </div>

    <div class="collapsible comment-content markdown-content">
      {{{text}}}
    </div>
  </div>
</div>
