<div id="hovercard">
  <a class='person_avatar'>
    <img class="avatar">
  </a>
  <h4>
    <a class="person"></a>
  </h4>
  <div class="handle"></div>
  <div id="hovercard_dropdown_container"></div>
  <div class="card-footer">
    <div class="footer-container">
      <div class="hashtags"></div>
    </div>
  </div>
</div>
