
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ stream_full_of_posts_q }}</h4>
    </a>
    <div class='answer hideable'>
      {{ stream_full_of_posts_a1 }}
      <ol>
        <li>{{ stream_full_of_posts_li1 }}</li>
        <li>{{ stream_full_of_posts_li2 }}</li>
        <li>{{ stream_full_of_posts_li3 }}</li>
      </ol>
    </div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ hide_posts_q }}</h4>
    </a>
    <div class='answer hideable'>{{ hide_posts_a }}</div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ post_notification_q }}</h4>
    </a>
    <div class='answer hideable'>{{ post_notification_a }}</div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ ignore_user_q }}</h4>
    </a>
    <div class='answer hideable'>
      <p>{{ ignore_user_a1 }}</p>
      <p>{{ ignore_user_a2 }}</p>
      <p>{{ ignore_user_a3 }}</p>
    </div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ post_report_q }}</h4>
    </a>
    <div class='answer hideable'>{{{ post_report_a }}}</div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ character_limit_q }}</h4>
    </a>
    <div class='answer hideable'>{{ character_limit_a }}</div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ char_limit_services_q }}</h4>
    </a>
    <div class='answer hideable'>{{ char_limit_services_a }}</div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ format_text_q }}</h4>
    </a>
    <div class='answer hideable'>{{{ format_text_a }}}</div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ insert_images_q }}</h4>
    </a>
    <div class='answer hideable'>{{ insert_images_a }}</div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ insert_images_comments_q }}</h4>
    </a>
    <div class='answer hideable'>
      {{ insert_images_comments_a }}
    </div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ size_of_images_q }}</h4>
    </a>
    <div class='answer hideable'>{{ size_of_images_a }}</div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ embed_multimedia_q }}</h4>
    </a>
    <div class='answer hideable'>{{ embed_multimedia_a }}</div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ post_location_q }}</h4>
    </a>
    <div class='answer hideable'>{{ post_location_a }}</div>
  </div>
  <div class='question opened collapsible'>
    <a class='toggle' href='#'>
      <h4>{{ post_poll_q }}</h4>
    </a>
    <div class='answer hideable'>{{ post_poll_a }}</div>
  </div>
