{{#if poll}}
  <div class="poll-form">
    <div class="poll-head">
      <strong>{{poll.question}}</strong>
      <div class="poll-statistic pull-right">
        {{t "poll.count" count=poll.participation_count}}
      </div>
    </div>
    <div class="poll-content">
      {{#if show_form}}
        <form action="/posts/{{poll.post_id}}/poll_participations" method="POST">
      {{/if}}
      {{#poll.poll_answers}}
        <div class="result-row">
          <div class="result-head">
            <input type="radio" name="vote" value="{{id}}"/>
            {{#if isCurrentUserVote}}
              <label>
                {{answer}}
                <span class="label label-primary">{{t "poll.your_vote"}}</span>
              </label>
            {{else}}
              <label>{{answer}}</label>
            {{/if}}
            <div class="poll-result pull-right">
              <span class="percentage"></span>
              ({{t "poll.answer_count" count=vote_count}})
            </div>
          </div>
          <div class="progress">
            {{#if isCurrentUserVote}}
              <div class="progress-bar users-vote" data-answerid="{{id}}" style="height: 100%"></div>
            {{else}}
              <div class="progress-bar" data-answerid="{{id}}" style="height: 100%"></div>
            {{/if}}
          </div>
        </div>
      {{/poll.poll_answers}}
      {{#if show_form}}
          <div class="toggle-result-wrapper">
            {{#if preview}}
              <span>{{t "poll.show_result"}}</span>
            {{else}}
              <a class="toggle-result" href="#">{{t "poll.show_result"}}</a>
            {{/if}}
          </div>
          <input type="submit" class="submit pull-right btn btn-default" value="{{t "poll.vote"}}"/>
          <div class="clearfix"></div>
        </form>
      {{/if}}

      {{#if is_reshare }}
        <div class="poll_footer">
          {{{t "poll.go_to_original_post" original_post_link=originalPostLink}}}
        </div>
      {{/if}}
    </div>
  </div>
{{/if}}
