.opengraph-container {
  width: 100%;
  display: block;
  text-decoration: none;
  font-style: normal;
  margin: 10px 0px 10px 0px;
  border-top: solid 1px $border-grey;
  border-bottom: solid 1px $border-grey;
  padding: 10px 0px 5px 0px;
  overflow: hidden;
  a {
    color: #000;
    .og-title {
      font-weight: bold;
    }
  }
  a:hover {
    color: $link-color;
  }

  .thumb {
    float: left;
    margin: 5px;
    margin-left: 0;
    max-width: 150px;
    padding-right: 5px;

    .video-overlay {
      cursor: pointer;
      height: 145px;
      width: 145px;
    }

    .overlay {
      background-color: rgba($black, .2);
      background-image: image-url('buttons/playbtn.png');
      background-position: center;
      background-repeat: no-repeat;
      background-size: 60px 60px;
      display: inline-block;
      height: 145px;
      position: relative;
      top: -145px;
      width: 145px;
    }
  }

  .og-description {
    color: $text-grey;
  }
}
