<a href="/tags/{{name}}" class="selectable">
  #{{ name }}
</a>
<a href="#" id="unfollow_{{name}}" rel="nofollow" class="action delete-tag-following pull-right" title="{{t "delete"}}">
  <i class="entypo-cross"></i>
</a>
