.card-footer {
  background-color: $background-grey;
  border-top: 1px solid $border-medium-grey;
  bottom: 0;
  font-size: $font-size-small;
  left: 0;
  line-height: $line-height-computed;
  margin-left: 0;
  min-height: $line-height-computed + 1;
  position: absolute;
  width: 100%;

  .footer-container {
    padding: 1px 5px;

    a {
      color: $text-grey;
      font-weight: normal;
      margin-right: 4px;
    }
  }
}

// gallery-picture specific styles for card-footer
.media {
  .gallery-picture img { margin-bottom: $line-height-computed + 2; }

  .card-footer .footer-container {  font-size: $font-size-base; }
}
