h2.help_header {
  border-bottom: 1px solid black;
  padding-bottom: 5px;
  margin-bottom: 15px;
}

ul#help_nav {
  display: inline;
  padding: 0;
  font-size: large;
  position: absolute;
  left: 198px;
  top: 5px;

  > li {
    display: inline;
    margin-right: 1em;
    font-weight: 700;

    a {
      padding: 2px;
      font-weight: normal;
    }
  }
}

#faq_nav {
  h4 {
    font-weight: 700;
  }
  ul {
    list-style: none;
    padding: 0;
    margin: 0;
    li {
      font-weight: 700;
      margin-bottom: 10px;
      a {
        font-weight: normal;
      }
      .section-selected {
        font-weight: bold;
      }
    }
  }
}

#faq {
  h1, h2 {
    text-align: center;
    font-weight: 700;
  }

  ul, ol {
    margin: 0;
    padding: 10px 0 10px 20px;
    text-align: left;
  }
  ul > li {
    list-style-type: disc !important;
  }

  text-align: justify;

  .question {
    background-color: rgb(242, 242, 242);
    margin-bottom: 10px;
    border-radius: 4px;
    a.toggle {
      text-decoration: none;
      color: black;
    }
    h4 {
      text-align: left;
      font-weight: 700;
      margin: 0px;
      padding: 10px 20px;
    }

    &.collapsed {
      border: 2px solid rgb(242, 242, 242);
    }

    &.opened {
      border: 2px solid rgb(142, 222, 61);
      h4 {
        background-color: rgb(142, 222, 61);
      }
    }
    .answer {
      border-radius: 0px 0px 4px 4px;
      background-color: white;
      padding: 10px 20px;

      div.help-chat-icons{
        text-align: center;
        font-size: 50px;
        line-height: 70px;

        [class^="entypo-"], [class*="entypo-"] {
          color: $text-color-pale;

          &.entypo-chat { color: $text-color-active; }
        }
      }
    }
  }
}
