.markdown-content {
  p { margin: 0 0 0.8em; }
  p:last-child { margin-bottom: 0; }

  ul, ol {
    margin-bottom: .8em;
    margin-top: .8em;
    li { margin-bottom: .5em; }
    &:first-child { margin-top: 0; }
    &:last-child { margin-bottom: 0; }
  }

  h1, h2, h3, h4, h5, h6 {
    &:first-child {
      margin-top: 0;
    }
  }

  .img-responsive {
    display: inline;
  }
}
