<li class="hoverable">
  <a class="selectable toggle_selector" href="#">
    {{ t "aspect_navigation.select_all" }}
  </a>
</li>
<li class="hoverable">
  <a href="#" class="selectable new_aspect" data-toggle="modal" data-target="#newAspectModal">
    {{ t "aspect_navigation.add_an_aspect" }}
  </a>
</li>
<div id="newAspectContainer"></div>
