
<table class="table">
  <thead>
    <tr>
      <th><i title="{{t 'admin.pods.ssl'}}" class="glyphicon glyphicon-lock"></i></th>
      <th>{{t 'admin.pods.pod'}}</th>
      <th class="added">{{t 'admin.pods.added'}}</th>
      <th>{{t 'admin.pods.status'}}</th>
      <th><i title="{{t 'admin.pods.actions'}}" class="entypo-tools pull-right"></i></th>
    </tr>
  </thead>
  <tbody>
  </tbody>
</table>
