<li>
  <form id="new_tag_following" accept-charset="UTF-8" action="/tag_followings" method="post">
    <input class="tag_input form-control" type="text" name="name" placeholder="{{t "stream.followed_tag.add_a_tag"}}" />
  </form>
</li>
