.stream-container {
  .stream-title {
    margin: 12px 0;
  }
}

.public-stream {
  float: none;
}

.main-stream-publisher {
  margin-top: 20px;
  padding: 0;

  .avatar {
    height: 50px;
    width: 50px;
  }

  .publisher {
    margin-left: 65px;
  }
}

@media(max-width: $screen-xs-max) {

  .main-stream-publisher .publisher {
    margin-left: 0;
  }
}
