<div class="poll-question form-group">
  <input class="input-block-level form-control" placeholder="{{t 'publisher.question' }}" type="text" name="poll_question">
</div>
<div class="poll-answers form-group clearfix row">
  <div class="poll-answer form-group clearfix">
    <div class="col-md-11">
      <input type="text" class="input-block-level form-control" name="poll_answers[]" placeholder="{{t 'publisher.option' }}">
    </div>
    <div class="col-md-1">
      <i class="remove-answer entypo-cross"></i>
    </div>
  </div>
  <div class="poll-answer form-group clearfix">
    <div class="col-md-11">
      <input type="text" class="input-block-level form-control" name="poll_answers[]" placeholder="{{t 'publisher.option' }}">
    </div>
    <div class="col-md-1">
      <i class="remove-answer entypo-cross"></i>
    </div>
  </div>
</div>
