<div class="reshare">
  <div class="media">
    {{#if root}}
      {{#with root}}
        <a href="/people/{{author.guid}}" class="img {{{hovercardable this}}}">
          {{{personImage author 'small'}}}
        </a>

        <div class="bd">
          <div>
            {{#linkToAuthor author}}
              {{name}}
            {{/linkToAuthor}}

            <span class="details gray">
              -
              <a href="/posts/{{id}}">
                <time class="timeago" data-original-title="{{{localTime created_at}}}" datetime="{{created_at}}" />
              </a>
            </span>
          </div>
      {{/with}}

        {{> status-message}}
      </div>
    {{else}}
      <p>
        {{t "stream.original_post_deleted"}}
      </p>
    {{/if}}
  </div>
</div>
