#invite_code {
  background-color: $framed-background;
  cursor: text;
  display: block;
  margin-top: 5px;
}

#invitationsModal {
  .modal-header, .modal-body {
    color: $text-color;
    font-size: $font-size-base;
    text-align: initial;
  }
  #paste_link { font-weight: 700; }
  #invite_code { margin-top: 10px; }
  #codes_left { color: $text-color-pale; }
  .controls { margin-left: 140px; }
  #email_invitation {
    padding-top: 10px;
    margin-top: 20px;
    border-top: 1px dashed $border-grey;
    label { font-weight: 700; }
    #already_sent {
      color: $text-color-pale;
      font-size: 12px;
    }
  }
}
