<div class="pull-right tag-following-action">
  <form accept-charset="UTF-8" action="/tag_followings" method="post">
      <input type="submit" class="btn
      {{#if tag_is_followed }}
        btn-success followed
      {{else}}
        btn-default
      {{/if}}
      " type="submit" value="{{followString}}"
        />
  </form>
</div>
